use uuid::Uuid;

use super::schema::{DbHook, OTP};

pub const METHOD_TOTP: &'static str = "totp";

/// Checks that the provided user has the specified OTP method
pub async fn has_otp(db: &mut DbHook, id: &Uuid, method: &str) -> sqlx::Result<bool> {
	Ok(sqlx::query!(
		"select otp_method from otp where user_id = $1 and otp_method = $2",
		id,
		method
	)
	.fetch_optional(&mut **db)
	.await?
	.is_some())
}

/// Lists the OTP methods the user has enabled
pub async fn list_enabled_methods(db: &mut DbHook, id: &Uuid) -> sqlx::Result<Vec<String>> {
	sqlx::query_scalar!("select otp_method from otp where user_id = $1", id)
		.fetch_all(&mut **db)
		.await
}

/// Gets the requested OTP method config, if set
pub async fn get_otp_method(db: &mut DbHook, id: &Uuid, method: &str) -> sqlx::Result<Option<OTP>> {
	sqlx::query_as!(
		OTP,
		"select * from otp where user_id = $1 and otp_method = $2",
		id,
		method
	)
	.fetch_optional(&mut **db)
	.await
}

/// Adds a new otp method with the provided config; will fail without check if one is already set
pub async fn add_otp_method(
	db: &mut DbHook,
	id: &Uuid,
	secret: &str,
	hashed_recovery_key: &str,
) -> sqlx::Result<()> {
	sqlx::query!(
		"insert into otp (user_id, otp_method, secret_seed, recovery_key) values ($1, $2, $3, $4)",
		id,
		METHOD_TOTP,
		secret,
		hashed_recovery_key,
	)
	.execute(&mut **db)
	.await?;

	Ok(())
}
