use std::net::IpAddr;

use qrcode_generator::QrCodeEcc;
use rocket::{
	http::{uri::Absolute, CookieJar},
	response::Redirect,
	serde::{json::Json, Serialize},
	State,
};
use rocket_dyn_templates::{context, Template};
use sqlx::Acquire;

use crate::{
	auth::session,
	config::Config,
	db::{
		doll,
		schema::{DollProfile, DollTagsDb, User},
		user,
	},
	pages::CommonTemplateState,
	routes::error_handlers::{PageResult, RawResult},
};

#[get("/")]
pub async fn index(mut db: DollTagsDb, user: User, meta: CommonTemplateState) -> PageResult {
	let tags = doll::list(&mut *db, &user.id).await?;
	let archived_tags = doll::list_archived(&mut *db, &user.id).await?;

	Ok(Template::render(
		"account/index",
		context! {
			meta,
			user,
			tags,
			archived_tags,
		},
	)
	.into())
}

#[get("/qr-png/<id>")]
pub fn qr_profile(id: &str, _user: User, config: &State<Config>) -> PageResult {
	let public_uri = Absolute::parse(&config.public_url)?;
	let built_uri = uri!(public_uri, crate::routes::public::short_url(id));
	let image = qrcode_generator::to_png_to_vec(built_uri.to_string(), QrCodeEcc::Low, 400)?;

	Ok(image.into())
}

#[derive(Debug, Serialize)]
#[serde(crate = "rocket::serde")]
pub struct DataDump {
	account: User,
	tags: Vec<DollProfile>,
	reserved_tags: Vec<i32>,
}

#[get("/data_dump")]
pub async fn export_data(
	mut db: DollTagsDb,
	user: User,
	client_ip: IpAddr,
) -> RawResult<Json<DataDump>> {
	let tags = doll::list_all(&mut *db, &user.id).await?;
	let reserved_tags = doll::list_archived(&mut *db, &user.id).await?;

	warn!(
		"[audit|{}] [{}] exported data",
		client_ip,
		user.id.to_string()
	);

	Ok(Json(DataDump {
		account: user,
		tags,
		reserved_tags,
	}))
}

#[get("/delete/<id>")]
pub async fn ask_delete(
	mut db: DollTagsDb,
	id: i32,
	user: User,
	meta: CommonTemplateState,
) -> PageResult {
	let db_tag = doll::get(&mut *db, id, "", false).await?;

	if let Some(tag) = db_tag {
		if tag.bound_to_id != user.id {
			Ok(Redirect::to(uri!("/account", index)).into())
		} else {
			Ok(Template::render(
				"tag/delete",
				context! {
					meta,
					tag,
				},
			)
			.into())
		}
	} else {
		Ok(Redirect::to(uri!("/account", index)).into())
	}
}

#[get("/yes_delete_this/<id>")]
pub async fn confirm_delete(
	mut db: DollTagsDb,
	id: i32,
	user: User,
	client_ip: IpAddr,
) -> PageResult {
	let mut trx = db.begin().await?;
	doll::delete(&mut trx, id, &user.id).await?;
	trx.commit().await?;

	warn!(
		"[audit|{}] [{}] deleted tag {:0>6}",
		client_ip,
		user.id.to_string(),
		id
	);

	Ok(Redirect::to(uri!("/account", index)).into())
}

#[get("/terminate")]
pub fn ask_terminate_account(_user: User, meta: CommonTemplateState) -> Template {
	Template::render("account/terminate", context! {meta})
}

#[get("/termin@or")]
pub async fn confirm_terminate_account(
	mut db: DollTagsDb,
	user: User,
	cookies: &CookieJar<'_>,
	client_ip: IpAddr,
) -> PageResult {
	let mut trx = db.begin().await?;
	doll::delete_all_from_account(&mut trx, &user.id).await?;
	user::delete(&mut trx, &user.id).await?;
	session::logout(cookies);
	trx.commit().await?;

	warn!(
		"[audit|{}] [{}] deleted account",
		client_ip,
		user.id.to_string()
	);

	Ok(Redirect::to("/").into())
}
