use std::net::IpAddr;

use rocket::{form::Form, http::CookieJar, response::Redirect, tokio::task};
use rocket_dyn_templates::{context, Template};
use totp_rs::Secret;

use crate::{
	auth::{self, pw},
	db::{
		self,
		otp::METHOD_TOTP,
		schema::{DollTagsDb, User},
	},
	ids::generate_recovery_key,
	pages::CommonTemplateState,
	routes::{self, error_handlers::PageResult},
};

#[get("/settings/totp")]
pub async fn show_totp_enable_start(
	mut db: DollTagsDb,
	user: User,
	cookies: &CookieJar<'_>,
	meta: CommonTemplateState,
) -> PageResult {
	if db::otp::has_otp(&mut *db, &user.id, METHOD_TOTP).await? {
		return Ok(Redirect::to(uri!("/account", routes::account::settings::show_settings)).into());
	}
	let totp_secret = Secret::generate_secret();
	auth::otp::cache_secret(cookies, &totp_secret);

	let totp = auth::otp::make_totp(&user.id.to_string(), totp_secret.to_bytes()?)?;
	let totp_qrcode = totp.get_qr_base64()?;

	Ok(Template::render(
		"account/otp/start",
		context! {
			meta,
			totp_qrcode,
			secret: totp.get_secret_base32(),
		},
	)
	.into())
}

#[derive(FromForm)]
pub struct OtpEnableForm {
	#[field(validate = len(6..=6))]
	pub otp_code: String,
}

#[post("/settings/totp", data = "<form>")]
pub async fn handle_totp_enable_start(
	mut db: DollTagsDb,
	form: Form<OtpEnableForm>,
	user: User,
	cookies: &CookieJar<'_>,
	meta: CommonTemplateState,
	client_ip: IpAddr,
) -> PageResult {
	if db::otp::has_otp(&mut *db, &user.id, METHOD_TOTP).await? {
		return Ok(Redirect::to(uri!("/account", routes::account::settings::show_settings)).into());
	}
	let secret = match auth::otp::get_secret(cookies) {
		Some(v) => v,
		None => return Ok(Redirect::to(uri!("/account", show_totp_enable_start)).into()),
	};

	let totp = auth::otp::make_totp(&user.id.to_string(), secret.to_bytes()?)?;

	if !totp.check_current(&form.otp_code)? {
		Err(String::from(
			"woof TODO need to impl. a user error return w/ the same secret",
		))?;
	}

	warn!(
		"[audit|{}] [{}] enabled TOTP 2FA",
		client_ip,
		user.id.to_string(),
	);

	let recovery_key = generate_recovery_key();

	{
		let recovery_key = recovery_key.clone();
		let hashed_recovery_key = task::spawn_blocking(move || pw::hash(&recovery_key)).await??;

		db::otp::add_otp_method(
			&mut *db,
			&user.id,
			&totp.get_secret_base32(),
			&hashed_recovery_key,
		)
		.await?;
	}

	Ok(Template::render(
		"account/otp/confirm",
		context! {
			meta,
			recovery_key,
		},
	)
	.into())
}
